@@ -1,7 +1,9 @@
 # Copyright (C) 2011-2012  Patrick Totzke <patricktotzke@gmail.com>
 # This file is released under the GNU GPL, version 3 or a later revision.
 # For further details see the COPYING file
-from alot.commands import Command, registerCommand
+from __future__ import absolute_import
+
+from ..commands import Command, registerCommand
 from . import globals
 
 MODE = 'bufferlist'
@@ -23,7 +25,7 @@ class BufferCloseCommand(Command):
         selected = bufferlist.get_selected_buffer()
         d = ui.apply_command(globals.BufferCloseCommand(buffer=selected))
 
-        def cb(ignoreme):
+        def cb(_):
             if bufferlist is not selected:
                 bufferlist.rebuild()
             ui.update()
