@@ -1,65 +1,96 @@
 # Copyright (C) 2011-2012  Patrick Totzke <patricktotzke@gmail.com>
 # This file is released under the GNU GPL, version 3 or a later revision.
 # For further details see the COPYING file
-import os
-import code
-from twisted.internet import threads
-from twisted.internet import defer
-import subprocess
-import email
-import urwid
-from twisted.internet.defer import inlineCallbacks
-import logging
+from __future__ import absolute_import
+from __future__ import division
+
 import argparse
+import code
+import email
+import email.utils
 import glob
+import logging
+import os
+import subprocess
 from StringIO import StringIO
 
-from alot.commands import Command, registerCommand
-from alot.completion import CommandLineCompleter
-from alot.commands import CommandParseError
-from alot.commands import commandfactory
-from alot.commands import CommandCanceled
-from alot import buffers
-from alot.widgets.utils import DialogBox
-from alot import helper
-from alot.db.errors import DatabaseLockedError
-from alot.completion import ContactsCompleter
-from alot.completion import AccountCompleter
-from alot.completion import TagsCompleter
-from alot.db.envelope import Envelope
-from alot import commands
-from alot.settings import settings
-from alot.helper import split_commandstring, split_commandline
-from alot.helper import mailto_to_envelope
-from alot.utils.booleanaction import BooleanAction
+import urwid
+from twisted.internet.defer import inlineCallbacks
+from twisted.internet import threads
+
+from . import Command, registerCommand
+from . import CommandCanceled
+from .utils import set_encrypt
+from .. import commands
+
+from .. import buffers
+from .. import helper
+from ..helper import split_commandstring
+from ..helper import mailto_to_envelope
+from ..completion import CommandLineCompleter
+from ..completion import ContactsCompleter
+from ..completion import AccountCompleter
+from ..completion import TagsCompleter
+from ..widgets.utils import DialogBox
+from ..db.errors import DatabaseLockedError
+from ..db.envelope import Envelope
+from ..settings.const import settings
+from ..utils import argparse as cargparse
 
 MODE = 'global'
 
 
 @registerCommand(MODE, 'exit')
 class ExitCommand(Command):
+    """Shut down cleanly.
+
+    The _prompt variable is for internal use only, it's used to control
+    prompting to close without sending, and is used by the BufferCloseCommand
+    if settings change after yielding to the UI.
+    """
+
+    def __init__(self, _prompt=True, **kwargs):
+        super(ExitCommand, self).__init__(**kwargs)
+        self.prompt_to_send = _prompt
 
-    """shut down cleanly"""
     @inlineCallbacks
     def apply(self, ui):
-        msg = 'index not fully synced. ' if ui.db_was_locked else ''
-        if settings.get('bug_on_exit') or ui.db_was_locked:
-            msg += 'really quit?'
+        if settings.get('bug_on_exit'):
+            msg = 'really quit?'
             if (yield ui.choice(msg, select='yes', cancel='no',
                                 msg_position='left')) == 'no':
                 return
+
+        # check if there are any unsent messages
+        if self.prompt_to_send:
+            for buffer in ui.buffers:
+                if (isinstance(buffer, buffers.EnvelopeBuffer) and
+                        not buffer.envelope.sent_time):
+                    msg = 'quit without sending message?'
+                    if (yield ui.choice(msg, cancel='no',
+                                        msg_position='left')) == 'no':
+                        raise CommandCanceled()
+
         for b in ui.buffers:
             b.cleanup()
-        ui.exit()
+        ui.apply_command(FlushCommand(callback=ui.exit))
+        ui.cleanup()
+
+        if ui.db_was_locked:
+            msg = 'Database locked. Exit without saving?'
+            if (yield ui.choice(msg, msg_position='left', cancel='no')) == 'no':
+                return
+            ui.exit()
 
 
 @registerCommand(MODE, 'search', usage='search query', arguments=[
     (['--sort'], {'help': 'sort order', 'choices': [
-                  'oldest_first', 'newest_first', 'message_id', 'unsorted']}),
+        'oldest_first', 'newest_first', 'message_id', 'unsorted']}),
     (['query'], {'nargs': argparse.REMAINDER, 'help': 'search string'})])
 class SearchCommand(Command):
 
-    """open a new search buffer"""
+    """open a new search buffer. Search obeys the notmuch
+    :ref:`search.exclude_tags <search.exclude_tags>` setting."""
     repeatable = True
 
     def __init__(self, query, sort=None, **kwargs):
@@ -114,12 +145,12 @@ class PromptCommand(Command):
         logging.info('open command shell')
         mode = ui.mode or 'global'
         cmpl = CommandLineCompleter(ui.dbman, mode, ui.current_buffer)
-        cmdline = yield ui.prompt('',
-                                  text=self.startwith,
-                                  completer=cmpl,
-                                  history=ui.commandprompthistory,
-                                  )
-        logging.debug('CMDLINE: %s' % cmdline)
+        cmdline = yield ui.prompt(
+            '',
+            text=self.startwith,
+            completer=cmpl,
+            history=ui.commandprompthistory)
+        logging.debug('CMDLINE: %s', cmdline)
 
         # interpret and apply commandline
         if cmdline:
@@ -141,14 +172,16 @@ class RefreshCommand(Command):
         ui.update()
 
 
-@registerCommand(MODE, 'shellescape', arguments=[
-    (['--spawn'], {'action': BooleanAction, 'default': None,
-                   'help': 'run in terminal window'}),
-    (['--thread'], {'action': BooleanAction, 'default': None,
-                    'help': 'run in separate thread'}),
-    (['--refocus'], {'action': BooleanAction, 'help': 'refocus current buffer \
-                     after command has finished'}),
-    (['cmd'], {'help': 'command line to execute'})],
+@registerCommand(
+    MODE, 'shellescape', arguments=[
+        (['--spawn'], {'action': cargparse.BooleanAction, 'default': None,
+                       'help': 'run in terminal window'}),
+        (['--thread'], {'action': cargparse.BooleanAction, 'default': None,
+                        'help': 'run in separate thread'}),
+        (['--refocus'], {'action': cargparse.BooleanAction,
+                         'help': 'refocus current buffer after command '
+                                 'has finished'}),
+        (['cmd'], {'help': 'command line to execute'})],
     forced={'shell': True},
 )
 class ExternalCommand(Command):
@@ -187,13 +220,13 @@ class ExternalCommand(Command):
         if touchhook is not None:
             logging.debug('calling hook: touch_external_cmdlist')
             res = touchhook(cmd, shell=shell, spawn=spawn, thread=thread)
-            logging.debug('got: %s' % res)
+            logging.debug('got: %s', res)
             cmd, shell, self.in_thread = res
         # otherwise if spawn requested and X11 is running
         elif spawn:
             if 'DISPLAY' in os.environ:
                 term_cmd = settings.get('terminal_cmd', '')
-                logging.info('spawn in terminal: %s' % term_cmd)
+                logging.info('spawn in terminal: %s', term_cmd)
                 termcmdlist = split_commandstring(term_cmd)
                 cmd = termcmdlist + cmd
             else:
@@ -208,7 +241,7 @@ class ExternalCommand(Command):
         Command.__init__(self, **kwargs)
 
     def apply(self, ui):
-        logging.debug('cmdlist: %s' % self.cmdlist)
+        logging.debug('cmdlist: %s', self.cmdlist)
         callerbuffer = ui.current_buffer
 
         # set standard input for subcommand
@@ -230,9 +263,9 @@ class ExternalCommand(Command):
                 logging.info('refocussing')
                 ui.buffer_focus(callerbuffer)
 
-        logging.info('calling external command: %s' % self.cmdlist)
+        logging.info('calling external command: %s', self.cmdlist)
 
-        def thread_code(*args):
+        def thread_code(*_):
             try:
                 if stdin is None:
                     proc = subprocess.Popen(self.cmdlist, shell=self.shell,
@@ -243,7 +276,7 @@ class ExternalCommand(Command):
                     proc = subprocess.Popen(self.cmdlist, shell=self.shell,
                                             stdin=subprocess.PIPE,
                                             stderr=subprocess.PIPE)
-                    out, err = proc.communicate(stdin.read())
+                    _, err = proc.communicate(stdin.read())
                     ret = proc.wait()
                 if ret == 0:
                     return 'success'
@@ -267,11 +300,6 @@ class ExternalCommand(Command):
             afterwards(ret)
 
 
-#@registerCommand(MODE, 'edit', arguments=[
-# (['--nospawn'], {'action': 'store_true', 'help':'spawn '}), #todo
-#    (['path'], {'help':'file to edit'})]
-#]
-#)
 class EditCommand(ExternalCommand):
 
     """edit a file"""
@@ -296,7 +324,7 @@ class EditCommand(ExternalCommand):
             editor_cmdstring = '/usr/bin/editor'
         editor_cmdstring = os.environ.get('EDITOR', editor_cmdstring)
         editor_cmdstring = settings.get('editor_cmd') or editor_cmdstring
-        logging.debug('using editor_cmd: %s' % editor_cmdstring)
+        logging.debug('using editor_cmd: %s', editor_cmdstring)
 
         self.cmdlist = None
         if '%s' in editor_cmdstring:
@@ -348,7 +376,7 @@ class RepeatCommand(Command):
     (['command'], {'help': 'python command string to call'})])
 class CallCommand(Command):
 
-    """ Executes python code """
+    """Executes python code"""
     repeatable = True
 
     def __init__(self, command, **kwargs):
@@ -362,10 +390,11 @@ class CallCommand(Command):
     def apply(self, ui):
         try:
             hooks = settings.hooks
-            env = {'ui': ui, 'settings': settings}
-            for k, v in env.items():
-                if k not in hooks.__dict__:
-                    hooks.__dict__[k] = v
+            if hooks:
+                env = {'ui': ui, 'settings': settings}
+                for k, v in env.iteritems():
+                    if k not in hooks.__dict__:
+                        hooks.__dict__[k] = v
 
             exec(self.command)
         except Exception as e:
@@ -375,10 +404,11 @@ class CallCommand(Command):
 
 
 @registerCommand(MODE, 'bclose', arguments=[
-    (['--redraw'], {'action': BooleanAction, 'help': 'redraw current buffer \
-                     after command has finished'}),
-    (['--force'], {'action': 'store_true',
-                   'help': 'never ask for confirmation'})])
+    (['--redraw'],
+     {'action': cargparse.BooleanAction,
+      'help': 'redraw current buffer after command has finished'}),
+    (['--force'],
+     {'action': 'store_true', 'help': 'never ask for confirmation'})])
 class BufferCloseCommand(Command):
 
     """close a buffer"""
@@ -398,24 +428,44 @@ class BufferCloseCommand(Command):
 
     @inlineCallbacks
     def apply(self, ui):
+        def one_buffer(prompt=True):
+            """Helper to handle the case on only one buffer being opened.
+
+            prompt is a boolean that is passed to ExitCommand() as the _prompt
+            keyword argument.
+            """
+            # If there is only one buffer and the settings don't allow using
+            # closebuffer to exit, then just stop.
+            if not settings.get('quit_on_last_bclose'):
+                msg = ('not closing last remaining buffer as '
+                       'global.quit_on_last_bclose is set to False')
+                logging.info(msg)
+                ui.notify(msg, priority='error')
+            # Otherwise pass directly to exit command, which also prommpts for
+            # 'close without sending'
+            else:
+                logging.info('closing the last buffer, exiting')
+                ui.apply_command(ExitCommand(_prompt=prompt))
+
         if self.buffer is None:
             self.buffer = ui.current_buffer
 
+        if len(ui.buffers) == 1:
+            one_buffer()
+            return
+
         if (isinstance(self.buffer, buffers.EnvelopeBuffer) and
                 not self.buffer.envelope.sent_time):
-            if (not self.force and (yield ui.choice('close without sending?',
-                                                    select='yes', cancel='no',
+            msg = 'close without sending?'
+            if (not self.force and (yield ui.choice(msg, cancel='no',
                                                     msg_position='left')) ==
                     'no'):
                 raise CommandCanceled()
 
+        # Because we yield above it is possible that the settings or the number
+        # of buffers chould change, so retest.
         if len(ui.buffers) == 1:
-            if settings.get('quit_on_last_bclose'):
-                logging.info('closing the last buffer, exiting')
-                ui.apply_command(ExitCommand())
-            else:
-                logging.info('not closing last remaining buffer as '
-                             'global.quit_on_last_bclose is set to False')
+            one_buffer(prompt=False)
         else:
             ui.buffer_close(self.buffer, self.redraw)
 
@@ -424,8 +474,9 @@ class BufferCloseCommand(Command):
                  help='focus previous buffer')
 @registerCommand(MODE, 'bnext', forced={'offset': +1},
                  help='focus next buffer')
-@registerCommand(MODE, 'buffer', arguments=[
-    (['index'], {'type': int, 'help': 'buffer index to focus'}), ],
+@registerCommand(
+    MODE, 'buffer',
+    arguments=[(['index'], {'type': int, 'help': 'buffer index to focus'})],
     help='focus buffer with given index')
 class BufferFocusCommand(Command):
 
@@ -467,7 +518,7 @@ class BufferFocusCommand(Command):
 class OpenBufferlistCommand(Command):
 
     """open a list of active buffers"""
-    def __init__(self, filtfun=None, **kwargs):
+    def __init__(self, filtfun=lambda x: x, **kwargs):
         """
         :param filtfun: filter to apply to displayed list
         :type filtfun: callable (str->bool)
@@ -490,7 +541,7 @@ class OpenBufferlistCommand(Command):
 class TagListCommand(Command):
 
     """opens taglist buffer"""
-    def __init__(self, filtfun=None, tags=None, **kwargs):
+    def __init__(self, filtfun=lambda x: x, tags=None, **kwargs):
         """
         :param filtfun: filter to apply to displayed list
         :type filtfun: callable (str->bool)
@@ -541,14 +592,15 @@ class FlushCommand(Command):
         except DatabaseLockedError:
             timeout = settings.get('flush_retry_timeout')
 
-            def f(*args):
-                self.apply(ui)
-            ui.mainloop.set_alarm_in(timeout, f)
-            if not ui.db_was_locked:
-                if not self.silent:
-                    ui.notify(
-                        'index locked, will try again in %d secs' % timeout)
-                ui.db_was_locked = True
+            if timeout > 0:
+                def f(*_):
+                    self.apply(ui)
+                ui.mainloop.set_alarm_in(timeout, f)
+                if not ui.db_was_locked:
+                    if not self.silent:
+                        ui.notify('index locked, will try again in %d secs'
+                                  % timeout)
+                    ui.db_was_locked = True
             ui.update()
             return
 
@@ -558,10 +610,8 @@ class FlushCommand(Command):
     (['commandname'], {'help': 'command or \'bindings\''})])
 class HelpCommand(Command):
 
-    """
-    display help for a command. Use \'bindings\' to
-    display all keybings interpreted in current mode.'
-    """
+    """display help for a command. Use \'bindings\' to display all keybings
+    interpreted in current mode.'"""
     def __init__(self, commandname='', **kwargs):
         """
         :param commandname: command to document
@@ -589,7 +639,7 @@ class HelpCommand(Command):
             if modemaps:
                 txt = (section_att, '\n%s-mode specific maps' % ui.mode)
                 linewidgets.append(urwid.Text(txt))
-                for (k, v) in modemaps.items():
+                for (k, v) in modemaps.iteritems():
                     line = urwid.Columns([('fixed', keycolumnwidth,
                                            urwid.Text((text_att, k))),
                                           urwid.Text((text_att, v))])
@@ -597,7 +647,7 @@ class HelpCommand(Command):
 
             # global maps
             linewidgets.append(urwid.Text((section_att, '\nglobal maps')))
-            for (k, v) in globalmaps.items():
+            for (k, v) in globalmaps.iteritems():
                 if k not in modemaps:
                     line = urwid.Columns(
                         [('fixed', keycolumnwidth, urwid.Text((text_att, k))),
@@ -617,7 +667,7 @@ class HelpCommand(Command):
                                     ('relative', 70))
             ui.show_as_root_until_keypress(overlay, 'esc')
         else:
-            logging.debug('HELP %s' % self.commandname)
+            logging.debug('HELP %s', self.commandname)
             parser = commands.lookup_parser(self.commandname, ui.mode)
             if parser:
                 ui.notify(parser.format_help(), block=True)
@@ -637,16 +687,17 @@ class HelpCommand(Command):
     (['--attach'], {'nargs': '+', 'help': 'attach files'}),
     (['--omit_signature'], {'action': 'store_true',
                             'help': 'do not add signature'}),
-    (['--spawn'], {'action': BooleanAction, 'default': None,
+    (['--spawn'], {'action': cargparse.BooleanAction, 'default': None,
                    'help': 'spawn editor in new terminal'}),
     (['rest'], {'nargs': '*'}),
 ])
 class ComposeCommand(Command):
 
     """compose a new email"""
-    def __init__(self, envelope=None, headers={}, template=None,
-                 sender=u'', subject=u'', to=[], cc=[], bcc=[], attach=None,
-                 omit_signature=False, spawn=None, rest=[], **kwargs):
+    def __init__(self, envelope=None, headers=None, template=None, sender=u'',
+                 subject=u'', to=None, cc=None, bcc=None, attach=None,
+                 omit_signature=False, spawn=None, rest=None, encrypt=False,
+                 **kwargs):
         """
         :param envelope: use existing envelope
         :type envelope: :class:`~alot.db.envelope.Envelope`
@@ -673,25 +724,28 @@ class ComposeCommand(Command):
         :param spawn: force spawning of editor in a new terminal
         :type spawn: bool
         :param rest: remaining parameters. These can start with
-                     'mailto' in which case it is interpreted sa mailto string.
+                     'mailto' in which case it is interpreted as mailto string.
                      Otherwise it will be interpreted as recipients (to) header
         :type rest: list(str)
+        :param encrypt: if the email should be encrypted
+        :type encrypt: bool
         """
 
         Command.__init__(self, **kwargs)
 
         self.envelope = envelope
         self.template = template
-        self.headers = headers
+        self.headers = headers or {}
         self.sender = sender
         self.subject = subject
-        self.to = to
-        self.cc = cc
-        self.bcc = bcc
+        self.to = to or []
+        self.cc = cc or []
+        self.bcc = bcc or []
         self.attach = attach
         self.omit_signature = omit_signature
         self.force_spawn = spawn
-        self.rest = ' '.join(rest)
+        self.rest = ' '.join(rest or [])
+        self.encrypt = encrypt
 
     @inlineCallbacks
     def apply(self, ui):
@@ -726,13 +780,14 @@ class ComposeCommand(Command):
                           priority='error')
                 return
             try:
-                self.envelope.parse_template(open(path).read())
+                with open(path) as f:
+                    self.envelope.parse_template(f.read())
             except Exception as e:
                 ui.notify(str(e), priority='error')
                 return
 
         # set forced headers
-        for key, value in self.headers.items():
+        for key, value in self.headers.iteritems():
             self.envelope.add(key, value)
 
         # set forced headers for separate parameters
@@ -748,81 +803,90 @@ class ComposeCommand(Command):
             self.envelope.add('Bcc', ','.join(self.bcc))
 
         # get missing From header
-        if not 'From' in self.envelope.headers:
+        if 'From' not in self.envelope.headers:
             accounts = settings.get_accounts()
             if len(accounts) == 1:
                 a = accounts[0]
-                fromstring = "%s <%s>" % (a.realname, a.address)
+                fromstring = email.utils.formataddr((a.realname, a.address))
                 self.envelope.add('From', fromstring)
             else:
                 cmpl = AccountCompleter()
                 fromaddress = yield ui.prompt('From', completer=cmpl,
-                                              tab=1)
+                                              tab=1, history=ui.senderhistory)
                 if fromaddress is None:
                     raise CommandCanceled()
 
+                ui.senderhistory.append(fromaddress)
                 self.envelope.add('From', fromaddress)
 
+        # find out the right account
+        sender = self.envelope.get('From')
+        name, addr = email.utils.parseaddr(sender)
+        account = settings.get_account_by_address(addr)
+        if account is None:
+            accounts = settings.get_accounts()
+            if not accounts:
+                ui.notify('no accounts set.', priority='error')
+                return
+            account = accounts[0]
+
         # add signature
-        if not self.omit_signature:
-            name, addr = email.Utils.parseaddr(self.envelope['From'])
-            account = settings.get_account_by_address(addr)
-            if account is not None:
-                if account.signature:
-                    logging.debug('has signature')
-                    sig = os.path.expanduser(account.signature)
-                    if os.path.isfile(sig):
-                        logging.debug('is file')
-                        if account.signature_as_attachment:
-                            name = account.signature_filename or None
-                            self.envelope.attach(sig, filename=name)
-                            logging.debug('attached')
-                        else:
-                            sigcontent = open(sig).read()
-                            enc = helper.guess_encoding(sigcontent)
-                            mimetype = helper.guess_mimetype(sigcontent)
-                            if mimetype.startswith('text'):
-                                sigcontent = helper.string_decode(sigcontent,
-                                                                  enc)
-                                self.envelope.body += '\n' + sigcontent
-                    else:
-                        ui.notify('could not locate signature: %s' % sig,
-                                  priority='error')
-                        if (yield ui.choice('send without signature?', 'yes',
-                                            'no')) == 'no':
-                            return
+        if not self.omit_signature and account.signature:
+            logging.debug('has signature')
+            sig = os.path.expanduser(account.signature)
+            if os.path.isfile(sig):
+                logging.debug('is file')
+                if account.signature_as_attachment:
+                    name = account.signature_filename or None
+                    self.envelope.attach(sig, filename=name)
+                    logging.debug('attached')
+                else:
+                    with open(sig) as f:
+                        sigcontent = f.read()
+                    enc = helper.guess_encoding(sigcontent)
+                    mimetype = helper.guess_mimetype(sigcontent)
+                    if mimetype.startswith('text'):
+                        sigcontent = helper.string_decode(sigcontent, enc)
+                        self.envelope.body += '\n' + sigcontent
+            else:
+                ui.notify('could not locate signature: %s' % sig,
+                          priority='error')
+                if (yield ui.choice('send without signature?', 'yes',
+                                    'no')) == 'no':
+                    return
 
         # Figure out whether we should GPG sign messages by default
         # and look up key if so
-        sender = self.envelope.get('From')
-        name, addr = email.Utils.parseaddr(sender)
-        account = settings.get_account_by_address(addr)
-        if account:
-            self.envelope.sign = account.sign_by_default
-            self.envelope.sign_key = account.gpg_key
+        if account.sign_by_default:
+            if account.gpg_key:
+                self.envelope.sign = account.sign_by_default
+                self.envelope.sign_key = account.gpg_key
+            else:
+                msg = 'Cannot find gpg key for account {}'.format(account.address)
+                logging.warn(msg)
+                ui.notify(msg, priority='error')
 
         # get missing To header
         if 'To' not in self.envelope.headers:
             allbooks = not settings.get('complete_matching_abook_only')
             logging.debug(allbooks)
-            if account is not None:
-                abooks = settings.get_addressbooks(order=[account],
-                                                   append_remaining=allbooks)
-                logging.debug(abooks)
-                completer = ContactsCompleter(abooks)
-            else:
-                completer = None
-            to = yield ui.prompt('To',
-                                 completer=completer)
+            abooks = settings.get_addressbooks(order=[account],
+                                               append_remaining=allbooks)
+            logging.debug(abooks)
+            completer = ContactsCompleter(abooks)
+            to = yield ui.prompt('To', completer=completer,
+                                 history=ui.recipienthistory)
             if to is None:
                 raise CommandCanceled()
 
-            self.envelope.add('To', to.strip(' \t\n,'))
+            to = to.strip(' \t\n,')
+            ui.recipienthistory.append(to)
+            self.envelope.add('To', to)
 
         if settings.get('ask_subject') and \
-                not 'Subject' in self.envelope.headers:
+                'Subject' not in self.envelope.headers:
             subject = yield ui.prompt('Subject')
-            logging.debug('SUBJECT: "%s"' % subject)
+            logging.debug('SUBJECT: "%s"', subject)
             if subject is None:
                 raise CommandCanceled()
 
@@ -831,7 +895,7 @@ class ComposeCommand(Command):
         if settings.get('compose_ask_tags'):
             comp = TagsCompleter(ui.dbman)
             tagsstring = yield ui.prompt('Tags', completer=comp)
-            tags = filter(lambda x: x, tagsstring.split(','))
+            tags = [t for t in tagsstring.split(',') if t]
             if tags is None:
                 raise CommandCanceled()
 
@@ -841,7 +905,22 @@ class ComposeCommand(Command):
             for gpath in self.attach:
                 for a in glob.glob(gpath):
                     self.envelope.attach(a)
-                    logging.debug('attaching: ' + a)
+                    logging.debug('attaching: %s', a)
+
+        # set encryption if needed
+        if self.encrypt or account.encrypt_by_default == u"all":
+            logging.debug("Trying to encrypt message because encrypt=%s and "
+                          "encrypt_by_default=%s", self.encrypt,
+                          account.encrypt_by_default)
+            yield set_encrypt(ui, self.envelope, block_error=self.encrypt)
+        elif account.encrypt_by_default == u"trusted":
+            logging.debug("Trying to encrypt message because "
+                          "account.encrypt_by_default=%s",
+                          account.encrypt_by_default)
+            yield set_encrypt(ui, self.envelope, block_error=self.encrypt, signed_only=True)
+        else:
+            logging.debug("No encryption by default, encrypt_by_default=%s",
+                          account.encrypt_by_default)
 
         cmd = commands.envelope.EditCommand(envelope=self.envelope,
                                             spawn=self.force_spawn,
@@ -849,11 +928,12 @@ class ComposeCommand(Command):
         ui.apply_command(cmd)
 
 
-@registerCommand(MODE, 'move', help='move focus in current buffer',
-                 arguments=[(['movement'], {
-                             'nargs': argparse.REMAINDER,
-                             'help': 'up, down, [half]page up, '
-                                     '[half]page down, first'})])
+@registerCommand(
+    MODE, 'move', help='move focus in current buffer',
+    arguments=[
+        (['movement'],
+         {'nargs': argparse.REMAINDER,
+          'help': 'up, down, [half]page up, [half]page down, first, last'})])
 class MoveCommand(Command):
 
     """move in widget"""
@@ -869,7 +949,7 @@ class MoveCommand(Command):
             ui.mainloop.process_input([self.movement])
         elif self.movement in ['halfpage down', 'halfpage up']:
             ui.mainloop.process_input(
-                ui.mainloop.screen_size[1] / 2 * [self.movement.split()[-1]])
+                ui.mainloop.screen_size[1] // 2 * [self.movement.split()[-1]])
         elif self.movement == 'first':
             if hasattr(ui.current_buffer, "focus_first"):
                 ui.current_buffer.focus_first()
@@ -881,3 +961,12 @@ class MoveCommand(Command):
         else:
             ui.notify('unknown movement: ' + self.movement,
                       priority='error')
+
+
+@registerCommand(MODE, 'reload', help='Reload all configuration files')
+class ReloadCommand(Command):
+
+    """Reload configuration."""
+
+    def apply(self, ui):
+        settings.reload()
