@@ -1,8 +1,10 @@
 # Copyright (C) 2011-2012  Patrick Totzke <patricktotzke@gmail.com>
 # This file is released under the GNU GPL, version 3 or a later revision.
 # For further details see the COPYING file
-from alot.commands import Command, registerCommand
-from alot.commands.globals import SearchCommand
+from __future__ import absolute_import
+
+from . import Command, registerCommand
+from .globals import SearchCommand
 
 MODE = 'taglist'
 
