@@ -3,7 +3,7 @@
 # For further details see the COPYING file
 
 
-class GPGCode:
+class GPGCode(object):
     AMBIGUOUS_NAME = 1
     NOT_FOUND = 2
     BAD_PASSPHRASE = 3
@@ -13,6 +13,8 @@ class GPGCode:
     KEY_CANNOT_ENCRYPT = 7
     KEY_CANNOT_SIGN = 8
     INVALID_HASH = 9
+    INVALID_HASH_ALGORITHM = 10
+    BAD_SIGNATURE = 11
 
 
 class GPGProblem(Exception):
