@@ -5,14 +5,17 @@
 """
 This contains alot-specific :class:`urwid.Widget` used in more than one mode.
 """
-import urwid
+from __future__ import absolute_import
+
+import functools
 import re
 import operator
+import urwid
 
-from alot.helper import string_decode
-from alot.settings import settings
-from alot.db.attachment import Attachment
-from alot.errors import CompletionError
+from ..helper import string_decode
+from ..settings.const import settings
+from ..db.attachment import Attachment
+from ..errors import CompletionError
 
 
 class AttachmentWidget(urwid.WidgetWrap):
@@ -51,7 +54,7 @@ class ChoiceWidget(urwid.Text):
         self.separator = separator
 
         items = []
-        for k, v in choices.items():
+        for k, v in choices.iteritems():
             if v == select and select is not None:
                 items += ['[', k, ']:', v]
             else:
@@ -84,7 +87,7 @@ class CompleteEdit(urwid.Edit):
 
     The interpretation of some keypresses is hard-wired:
         :enter: calls 'on_exit' callback with current value
-        :esc: calls 'on_exit' with value `None`, which can be interpreted
+        :esc/ctrl g: calls 'on_exit' with value `None`, which can be interpreted
               as cancelation
         :tab: calls the completer and tabs forward in the result list
         :shift tab: tabs backward in the result list
@@ -123,6 +126,7 @@ class CompleteEdit(urwid.Edit):
         self.on_error = on_error
         self.history = list(history)  # we temporarily add stuff here
         self.historypos = None
+        self.focus_in_clist = 0
 
         if not isinstance(edit_text, unicode):
             edit_text = string_decode(edit_text)
@@ -140,7 +144,7 @@ class CompleteEdit(urwid.Edit):
                     self.completions += self.completer.complete(self.edit_text,
                                                                 self.edit_pos)
                     self.focus_in_clist = 1
-                except CompletionError, e:
+                except CompletionError as e:
                     if self.on_error is not None:
                         self.on_error(e)
 
@@ -168,7 +172,7 @@ class CompleteEdit(urwid.Edit):
                 self.set_edit_text(self.history[self.historypos])
         elif key == 'enter':
             self.on_exit(self.edit_text)
-        elif key == 'esc':
+        elif key in ('ctrl g', 'esc'):
             self.on_exit(None)
         elif key == 'ctrl a':
             self.set_edit_pos(0)
@@ -268,12 +272,17 @@ class HeadersList(urwid.WidgetWrap):
         return headerlines
 
 
+@functools.total_ordering
 class TagWidget(urwid.AttrMap):
     """
     text widget that renders a tagstring.
 
     It looks up the string it displays in the `tags` section
     of the config as well as custom theme settings for its tag.
+
+    Atributes that should be considered publicly readable:
+        :attr tag: the notmuch tag
+        :type tag: str
     """
     def __init__(self, tag, fallback_normal=None, fallback_focus=None):
         self.tag = tag
@@ -302,11 +311,35 @@ class TagWidget(urwid.AttrMap):
     def keypress(self, size, key):
         return key
 
-    def get_tag(self):
-        return self.tag
-
     def set_focussed(self):
-        self.set_attr_map(self.attmap['focus'])
+        self.set_attr_map(self.attmaps['focus'])
 
     def set_unfocussed(self):
-        self.set_attr_map(self.attmap['normal'])
+        self.set_attr_map(self.attmaps['normal'])
+
+    def __lt__(self, other):
+        """Groups tags of 1 character first, then alphabetically.
+
+        This groups tags unicode characters at the begnining.
+        """
+        if not isinstance(other, TagWidget):
+            return NotImplemented
+
+        self_len = len(self.translated)
+        oth_len = len(other.translated)
+
+        if (self_len == 1) is not (oth_len == 1):
+            return self_len < oth_len
+        return self.translated.lower() < other.translated.lower()
+
+    def __eq__(self, other):
+        if not isinstance(other, TagWidget):
+            return NotImplemented
+        if len(self.translated) != len(other.translated):
+            return False
+        return self.translated.lower() == other.translated.lower()
+
+    def __ne__(self, other):
+        if not isinstance(other, TagWidget):
+            return NotImplemented
+        return self.translated.lower() != other.translated.lower()
