@@ -5,6 +5,8 @@
 """
 Utility Widgets not specific to alot
 """
+from __future__ import absolute_import
+
 import urwid
 
 
@@ -33,8 +35,7 @@ class DialogBox(urwid.WidgetWrap):
                             align='center',
                             valign='top',
                             width=len(title),
-                            height=None,
-                            )
+                            height=None)
         urwid.WidgetWrap.__init__(self, box)
 
     def selectable(self):
